/*
 **  EPIwm
 **  Copyright (C) 1999 Julien Mulot, Thomas Poindessous, Stephane Rault
 **
 **  This program is free software; you can redistribute it and/or modify
 **  it under the terms of the GNU General Public License as published by
 **  the Free Software Foundation; either version 2 of the License, or
 **  (at your option) any later version.
 **
 **  This program is distributed in the hope that it will be useful,
 **  but WITHOUT ANY WARRANTY; without even the implied warranty of
 **  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 **  GNU General Public License for more details.
 **
 **  You should have received a copy of the GNU General Public License
 **  along with this program; if not, write to the Free Software
 **  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 **
 ** window.c
 ** 
 ** Made by (Julien Mulot)
 ** Login   <mulot@epita.fr>
 ** 
 ** Started on  Fri Jan  8 21:13:01 1999 Julien Mulot
 ** Last update Mon May 10 22:32:58 1999 Poindessous Thomas
 */

#include "epiwm.h"
#include <X11/Xutil.h>
#include <X11/extensions/shape.h>

int		test_shape(w)
Window		w;
{
   int		xws;
   int		yws;
   int		xbs;
   int		ybs;
   unsigned	wws;
   unsigned	hws;
   unsigned	wbs;
   unsigned	hbs;
   int		boundingShaped;
   int		clipShaped;
   
   XShapeSelectInput(gl_dpy, w, ShapeNotifyMask);
   XShapeQueryExtents(gl_dpy, w, &boundingShaped, &xws, &yws, &wws,
		      &hws, &clipShaped, &xbs, &ybs, &wbs, &hbs);
   return (boundingShaped);
}

void			add_shape(win, parent)
Window			win;
Window			parent;
{
   XRectangle		rect[4];
   XWindowAttributes	attrib;
   unsigned short	nb_rect;
   /* nb_rect nb de cotes */
   nb_rect = 4;
   XGetWindowAttributes(gl_dpy, parent, &attrib);
   rect[0].x = 0;
   rect[0].y = 0;
   rect[0].width = attrib.width;
   rect[0].height = gl_epiwin.border_height + gl_epiwin.titlebar.height +
     gl_epiwin.titlebar.border*2;
   rect[1].x = 0;
   rect[1].y = 0;
   rect[1].width = gl_epiwin.border_width + gl_epiwin.bf_border;
   rect[1].height = attrib.height;
   rect[2].x = attrib.width - gl_epiwin.border_width - gl_epiwin.bf_border;
   rect[2].y = 0;
   rect[2].width = gl_epiwin.border_width + gl_epiwin.bf_border;
   rect[2].height = attrib.height;
   rect[3].x = 0;
   rect[3].y = attrib.height - gl_epiwin.border_height - gl_epiwin.bf_border;
   rect[3].width = attrib.width;
   rect[3].height = gl_epiwin.border_height + gl_epiwin.bf_border;   
   XGetWindowAttributes(gl_dpy, win, &attrib);
   XShapeCombineShape(gl_dpy, parent, ShapeBounding,
		      gl_epiwin.border_width + attrib.border_width
		      + gl_epiwin.bf_border,
		      gl_epiwin.border_height + attrib.border_width
		      + gl_epiwin.titlebar.height
		      + 2*gl_epiwin.titlebar.border,
		      win, ShapeBounding, ShapeSet);
   XShapeCombineRectangles(gl_dpy, parent, ShapeBounding,
			   0, 0, rect, nb_rect,
			   ShapeUnion, Unsorted);     
}

void			add_border(win)
Window			win;
{     
   t_win		new_win;
   t_win		*temp;
   XWindowAttributes	attrib;
   XSetWindowAttributes	attr;  
   char			*name;
   XWMHints		*wmhints;
   t_mouse_move_win	*list;
   
   XGetWindowAttributes(gl_dpy, win, &attrib);
   if (!attrib.override_redirect)
     {
	XAddToSaveSet(gl_dpy, win);
	XGrabServer(gl_dpy);
	XUnmapWindow(gl_dpy, win);
	new_win.win_border = attrib.border_width;
	new_win.win = win;
	check_properties(&new_win);
	new_win.is_win_shape = test_shape(new_win.win);
	new_win.parent = add_parent(&attrib);
	XFetchName(gl_dpy, win, &name);
	XStoreName(gl_dpy, new_win.parent, name);
	new_win.border_frame = add_border_frame(new_win.parent);
	new_win.titlebar_win = add_titlebar_win(new_win.parent);
	XStoreName(gl_dpy, new_win.titlebar_win, name);
	free(name);
	XGetIconName(gl_dpy, win, &name);
	XSetIconName(gl_dpy, new_win.parent, name);
	free(name);
	if (gl_epiwin.is_btn_resize)
	  new_win.button_resize = add_resize_btn(new_win.titlebar_win);
	if (gl_epiwin.is_btn_max)
	  new_win.button_max = add_resize_max(new_win.titlebar_win);  
	if (gl_epiwin.is_btn_iconify)
	  new_win.button_iconify = add_iconify_btn(new_win.titlebar_win);
	if (gl_epiwin.is_btn_shade)
	  new_win.button_iconify = add_shade_btn(new_win.titlebar_win);
	new_win.has_parent = 1;      
	add_elt(&new_win);
	attr.event_mask = PropertyChangeMask /* | ResizeRedirectMask */
	  | StructureNotifyMask;
	/*attr.do_not_propagate_mask = PointerMotionMask;*/
	XChangeWindowAttributes(gl_dpy, new_win.win,
				CWEventMask /* | CWDontPropagate*/, &attr);
	XReparentWindow(gl_dpy, win, new_win.border_frame,
			gl_epiwin.border_width, gl_epiwin.border_height);
	wmhints = XGetWMHints(gl_dpy, win);
	list = NULL;
	temp = NULL;
	if (wmhints)
	  {
	     if (wmhints->initial_state == IconicState)
	       {
		  list = malloc(sizeof(t_mouse_move_win));
		  list->button = malloc(sizeof(XButtonEvent));
		  list->button->x = 0;
		  list->button->y = 0;
		  list->temp = search_elt(new_win.win);
		  if (list->temp && wmhints->icon_window)
		    list->temp->icon_choose_win = wmhints->icon_window;
		  mouse_iconify(list);
		  free(list->button);
		  free(list);
	       }
	     else
	       if (attrib.map_state == IsViewable)
		 {
		    window_group_map(&new_win);
		    window_state(&new_win, "inactive");
		    catch_win_mouse();
		 }  	     
	  }
	else
	  if (attrib.map_state == IsViewable)
	    {
	       window_group_map(&new_win);
	       window_state(&new_win, "inactive");
	       catch_win_mouse();
	    }
	XUngrabServer(gl_dpy);
	XSync(gl_dpy, 1);
     }
}

void		handle_window(win)
Window		win;
{
   t_win	*tmp;
   
   add_border(win);
   tmp = search_elt(win);
   if (tmp)
     workspace_window_new(gl_epiwin.first_workspace, tmp);
}

void		create_win(win)
Window		win;
{
   t_win	*tmp;
   
   add_border(win);
   tmp = search_elt(win);
   if (tmp)
     workspace_window_new(gl_epiwin.current_workspace, tmp);
}

Window			add_parent(attrib)
XWindowAttributes	*attrib;
{
   Window		temp;
   XSetWindowAttributes	attr;
   
   temp = XCreateSimpleWindow(gl_dpy,
			      gl_scr_info.root_win,
			      attrib->x,	   
			      attrib->y,  
			      attrib->width +
			      (gl_epiwin.border_width +
			       attrib->border_width +
			       gl_epiwin.bf_border)*2,
			      attrib->height +
			      (gl_epiwin.border_height +
			       attrib->border_width +
			       gl_epiwin.titlebar.border)*2
			      + gl_epiwin.titlebar.height
			      + gl_epiwin.bf_border,
			      0,
			      BlackPixel(gl_dpy, gl_scr_info.screen),
			      WhitePixel(gl_dpy, gl_scr_info.screen));
   attr.event_mask = ExposureMask | EnterWindowMask | LeaveWindowMask
     | ButtonPressMask | ButtonReleaseMask | SubstructureNotifyMask
       | PropertyChangeMask | SubstructureRedirectMask;
   XChangeWindowAttributes(gl_dpy, temp , CWEventMask, &attr);
   return (temp);
}

Window			add_resize_btn(parent)
Window			parent;
{
   t_window_btn		tmp;
   XWindowAttributes	attrib;
   XSetWindowAttributes	attr;  
   Window		w_tmp;
   
   XGetWindowAttributes(gl_dpy, parent, &attrib);
   /* coin haut droit */
   tmp.x = set_btn_position(parent, gl_epiwin.align_btn_resize,
			    gl_epiwin.pos_btn_resize);
   tmp.y = (gl_epiwin.titlebar.height - gl_epiwin.btn_height) / 2;
   tmp.width = gl_epiwin.btn_width;
   tmp.height = gl_epiwin.btn_height ;
   tmp.bgcolor = BlackPixel(gl_dpy, gl_scr_info.screen);
   tmp.fgcolor = gl_epiwin.titlebar.color_active.pixel;
   tmp.parent = parent;
   tmp.cursor = gl_epiwin.normal_cursor;
   tmp.border = 0;
   w_tmp = add_btn(&tmp);
   attr.event_mask = ButtonPressMask | ButtonReleaseMask | ExposureMask;
   XChangeWindowAttributes(gl_dpy, w_tmp, CWEventMask, &attr);
   return (w_tmp);
}

Window			add_resize_max(parent)
Window			parent;
{
   t_window_btn		tmp;
   XWindowAttributes	attrib;
   XSetWindowAttributes	attr;
   Window		w_tmp;
   
   XGetWindowAttributes(gl_dpy, parent, &attrib);
   /* coin haut droit 2 ieme en retrait */
   tmp.width = gl_epiwin.btn_width;
   tmp.height = gl_epiwin.btn_height;
   tmp.x = set_btn_position(parent, gl_epiwin.align_btn_max,
			    gl_epiwin.pos_btn_max);
   tmp.y = (gl_epiwin.titlebar.height - gl_epiwin.btn_height) / 2;
   tmp.bgcolor = BlackPixel(gl_dpy, gl_scr_info.screen);
   tmp.fgcolor = gl_epiwin.titlebar.color_active.pixel;
   tmp.parent = parent;
   tmp.cursor = gl_epiwin.resize_NE_cursor;
   tmp.border = 0;
   w_tmp = add_btn(&tmp);
   attr.event_mask = ButtonPressMask | ButtonReleaseMask | ExposureMask;
   XChangeWindowAttributes(gl_dpy, w_tmp, CWEventMask, &attr);
   return (w_tmp);
}

Window			add_iconify_btn(parent)
Window			parent;
{
   t_window_btn		tmp;
   XSetWindowAttributes	attr;
   Window		w_tmp;   
   
   tmp.width = gl_epiwin.btn_width;
   tmp.height = gl_epiwin.btn_height;
   tmp.x = set_btn_position(parent, gl_epiwin.align_btn_iconify,
			    gl_epiwin.pos_btn_iconify);
   tmp.y = (gl_epiwin.titlebar.height - gl_epiwin.btn_height) / 2;
   tmp.bgcolor = BlackPixel(gl_dpy, gl_scr_info.screen);
   tmp.fgcolor = gl_epiwin.titlebar.color_active.pixel;
   tmp.parent = parent;
   tmp.cursor = gl_epiwin.iconify_cursor;
   tmp.border = 0;
   w_tmp = add_btn(&tmp);
   attr.event_mask = ButtonPressMask | ButtonReleaseMask | ExposureMask;
   XChangeWindowAttributes(gl_dpy, w_tmp, CWEventMask, &attr);
   return (w_tmp);
}

Window			add_shade_btn(parent)
Window			parent;
{
   t_window_btn		tmp;
   XSetWindowAttributes	attr;
   Window		w_tmp;   
   
   tmp.width = gl_epiwin.btn_width;
   tmp.height = gl_epiwin.btn_height;
   tmp.x = set_btn_position(parent, gl_epiwin.align_btn_shade,
			    gl_epiwin.pos_btn_shade);
   tmp.y = (gl_epiwin.titlebar.height - gl_epiwin.btn_height) / 2;
   tmp.bgcolor = BlackPixel(gl_dpy, gl_scr_info.screen);
   tmp.fgcolor = gl_epiwin.titlebar.color_active.pixel;
   tmp.parent = parent;
   tmp.cursor = gl_epiwin.iconify_cursor;
   tmp.border = 0;
   w_tmp = add_btn(&tmp);
   attr.event_mask = ButtonPressMask | ButtonReleaseMask | ExposureMask;
   XChangeWindowAttributes(gl_dpy, w_tmp, CWEventMask, &attr);
   return (w_tmp);
}

Window			add_border_frame(parent)
Window			parent;
{
   t_window_btn		tmp;
   XWindowAttributes	attrib;
   XSetWindowAttributes	attr;
   Window		w_tmp;
   
   XGetWindowAttributes(gl_dpy, parent, &attrib);
   tmp.bgcolor = BlackPixel(gl_dpy, gl_scr_info.screen);
   tmp.fgcolor = gl_epiwin.color_active.pixel;
   tmp.parent = parent;
   tmp.cursor = 0;
   tmp.border = gl_epiwin.bf_border;
   tmp.x = 0;
   tmp.y = gl_epiwin.titlebar.height + gl_epiwin.titlebar.border*2
     - gl_epiwin.bf_border;
   tmp.width = attrib.width - gl_epiwin.bf_border*2 ;
   tmp.height = attrib.height - gl_epiwin.titlebar.height
     - (gl_epiwin.titlebar.border*2) - gl_epiwin.bf_border;
   w_tmp = add_btn(&tmp);
   attr.event_mask = ExposureMask | ButtonPressMask | ButtonReleaseMask
     | SubstructureRedirectMask | SubstructureNotifyMask | PointerMotionMask;
   /*
      attr.do_not_propagate_mask = ButtonPressMask | ButtonReleaseMask;
      */
   /*   | CWDontPropagate */
   XChangeWindowAttributes(gl_dpy, w_tmp, CWEventMask, &attr);
   return (w_tmp);
}

Window			add_titlebar_win(parent)
Window			parent;
{
   t_window_btn		tmp;
   XWindowAttributes	attrib;
   XSetWindowAttributes	attr;
   Window		w_tmp;
   
   XGetWindowAttributes(gl_dpy, parent, &attrib);
   tmp.x = 0;
   tmp.y = 0;
   tmp.bgcolor = BlackPixel(gl_dpy, gl_scr_info.screen);
   tmp.fgcolor = gl_epiwin.titlebar.color_active.pixel;
   tmp.parent = parent;
   tmp.cursor = 0;
   tmp.border = gl_epiwin.titlebar.border;
   tmp.width = attrib.width - 2*gl_epiwin.titlebar.border;
   tmp.height = gl_epiwin.titlebar.height;
   w_tmp = add_btn(&tmp);
   attr.event_mask = ButtonPressMask | ButtonReleaseMask
     | ExposureMask | PointerMotionMask;
   XChangeWindowAttributes(gl_dpy, w_tmp, CWEventMask, &attr);
   return (w_tmp);
}


Window		add_btn(win_struct)
t_window_btn	*win_struct;
{
   Window	temp;
   
   temp = XCreateSimpleWindow(gl_dpy,
			      win_struct->parent,
			      win_struct->x,
			      win_struct->y,
			      win_struct->width,
			      win_struct->height,
			      win_struct->border,
			      win_struct->bgcolor,
			      win_struct->fgcolor);
   if (win_struct->cursor)
     XDefineCursor(gl_dpy, temp, win_struct->cursor);
   return (temp);
}

void		check_properties(wg_tmp)
t_win		*wg_tmp;
{
   long		toto;
   
   XGetWMNormalHints(gl_dpy, wg_tmp->win, &wg_tmp->hints, &toto);
   if (wg_tmp->hints.flags & PResizeInc)
     {
	if (wg_tmp->hints.width_inc == 0)
	  wg_tmp->hints.width_inc = 1;
	if (wg_tmp->hints.height_inc == 0)
	  wg_tmp->hints.height_inc = 1;
     }
   else
     {
	wg_tmp->hints.width_inc = 1;
	wg_tmp->hints.height_inc = 1;
     }
   if (wg_tmp->hints.flags & PMinSize)
     {
	if (wg_tmp->hints.min_width  == 0)
	  wg_tmp->hints.min_width = 1;
	if (wg_tmp->hints.min_height  == 0)
	  wg_tmp->hints.min_height = 1;
     }
   else
     {
	wg_tmp->hints.min_width = 1;
	wg_tmp->hints.min_height = 1;
     }
   if (!(wg_tmp->hints.flags & PMaxSize))
     {	
	wg_tmp->hints.max_width = 0;
	wg_tmp->hints.max_height = 0;
     }
}


