/*
**  EPIwm
**  Copyright (C) 1999 Julien Mulot, Thomas Poindessous, Stephane Rault
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**
** window_draw.c
** 
** Made by (mulot)
** Login   <mulot_j@epita.fr>
** 
** Started on  Thu Apr  8 21:35:18 1999 mulot
 ** Last update Tue May 18 16:53:22 1999 Thomas Poindessous
*/

#include "epiwm.h"

void	set_background(button, state)
     Window	button;
     char	*state;
{
  if (!strcmp("active", state))
    XSetWindowBackground(gl_dpy, button,
			 gl_epiwin.titlebar.color_active.pixel);
  else
    XSetWindowBackground(gl_dpy, button,
			 gl_epiwin.titlebar.color_inactive.pixel);
  XClearWindow(gl_dpy, button);
}


void	titlebar_button_state(win, state)
     t_win	*win;
     char	*state;
{
  if (gl_epiwin.is_btn_iconify)
    {
      set_background(win->button_iconify, state);
      draw_icon_iconify(win->button_iconify, state);
    }
  if (gl_epiwin.is_btn_max)
    {
      set_background(win->button_max, state);
      draw_icon_max(win->button_max, state);
    }
  if (gl_epiwin.is_btn_resize)
    {
      set_background(win->button_resize, state);
      draw_icon_resize(win->button_resize, state);
    }
  if (gl_epiwin.is_btn_close)
    {
      set_background(win->button_close, state);
      draw_icon_close(win->button_close, state);
    }
  if (gl_epiwin.is_btn_shade)
    {
      set_background(win->button_shade, state);
      draw_icon_shade(win->button_shade, state);
    }
}

void	window_state(win, state)
     t_win	*win;
     char	*state;
{
  if (!strcmp(state, "active"))
    {
       if (gl_epiwin.current)
	 if (gl_epiwin.current != win)
	   window_state(gl_epiwin.current, "inactive");
#ifdef	  __Imlib__
      if (gl_epiwin.titlebar.image_active)
	draw_titlebar_image(win->titlebar_win, state);
      else
	{
#endif
	  if (gl_epiwin.titlebar.grad_active_type)
	    draw_gradient(win->titlebar_win, "titlebar", state);
	  else
	    {
	      XSetWindowBackground(gl_dpy, win->titlebar_win, 
				   gl_epiwin.titlebar.color_active.pixel);
	      XClearWindow(gl_dpy, win->titlebar_win);
	    }
#ifdef	  __Imlib__
	}
#endif
      XSetWindowBackground(gl_dpy, win->border_frame, 
			   gl_epiwin.color_active.pixel);
      XClearWindow(gl_dpy, win->border_frame);
      gl_epiwin.current = win;
      win->active = 1;
    }
  else
    {
#ifdef	  __Imlib__
      if (gl_epiwin.titlebar.image_inactive)
	draw_titlebar_image(win->titlebar_win, state);
      else
	{
#endif
	  if (gl_epiwin.titlebar.grad_inactive_type)
	    draw_gradient(win->titlebar_win, "titlebar", state);
	  else
	    {
	      XSetWindowBackground(gl_dpy, win->titlebar_win, 
				   gl_epiwin.titlebar.color_inactive.pixel);
	      XClearWindow(gl_dpy, win->titlebar_win);
	    }
#ifdef	  __Imlib__
	}
#endif
      XSetWindowBackground(gl_dpy, win->border_frame, 
			   gl_epiwin.color_inactive.pixel);
      XClearWindow(gl_dpy, win->border_frame);
      win->active = 0;
    }
  titlebar_button_state(win, state);
  draw_titlebar(win->titlebar_win, state);
}

/* write application's name on titlebar */
void			draw_titlebar(titlebar, state)
     Window		titlebar;
     char		*state;
{
#define SPACE 18
  char			*name;
  int			font_direction;
  int			top_size;
  int			bottom_size;
  int			start;
  XCharStruct		dim;
  XWindowAttributes	attrib;
  
  XGetWindowAttributes(gl_dpy, titlebar, &attrib);
  XFetchName(gl_dpy, titlebar, &name);
  XTextExtents(gl_epiwin.titlebar.font, name, strlen(name), &font_direction,
	       &top_size, &bottom_size, &dim);
  if (!strcmp(gl_epiwin.titlebar.align, "right"))
    start = attrib.width - gl_epiwin.titlebar.space - dim.width;
  else if (!strcmp(gl_epiwin.titlebar.align, "center"))
    start = ((attrib.width - dim.width) / 2);
  else
    start = gl_epiwin.titlebar.space;
  if (!strcmp(state, "active"))
    XDrawString(gl_dpy, titlebar, gl_epiwin.titlebar.font_active,
		start, gl_epiwin.titlebar.height / 2 + top_size / 2, 
		name, strlen(name));
  else
    XDrawString(gl_dpy, titlebar, gl_epiwin.titlebar.font_inactive,
		start, gl_epiwin.titlebar.height / 2 + top_size / 2, 
		name, strlen(name));
  free(name);
}

#ifdef	  __Imlib__
void		draw_titlebar_image(titlebar, state)
     Window          titlebar;
     char		*state;
{
  int		width;
  int		height;
  Pixmap	pixmap;
  Pixmap	mask;
  
  if (!strcmp(state, "active"))
    if (!(gl_epiwin.titlebar.image =
	  Imlib_load_image(gl_imlib_data, gl_epiwin.titlebar.image_active)))
      {
	fprintf(stderr, "cannot load titlebar active image\n");
	exit (1);
      }
    else
      if (!(gl_epiwin.titlebar.image =
	    Imlib_load_image(gl_imlib_data,
			     gl_epiwin.titlebar.image_inactive)))
	{
	  fprintf(stderr, "cannot load titlebar inactive image\n");
	  exit (1);
	}
  width = gl_epiwin.titlebar.image->rgb_width;
  height = gl_epiwin.titlebar.image->rgb_height;
  Imlib_render(gl_imlib_data, gl_epiwin.titlebar.image, 
	       width, height);  
  pixmap = Imlib_move_image(gl_imlib_data, 
			    gl_epiwin.titlebar.image);
  mask = Imlib_move_mask(gl_imlib_data, 
			 gl_epiwin.titlebar.image);
  XSetWindowBackgroundPixmap(gl_dpy, titlebar, pixmap);
  XClearWindow(gl_dpy, titlebar);
}
#endif

Pixel	*AllocLinearGradient(char *s_from, char *s_to, int npixels)
{
  Pixel *pixels;
  XColor from, to, c;
  int r, dr, g, dg, b, db;
  int i = 0, got_all = 1;
  
  if (npixels < 1) 
    return NULL;
  if (!s_from || !XParseColor(gl_dpy, 
			      XDefaultColormap(gl_dpy, gl_scr_info.screen),
			      s_from, &from))
    return NULL;
  if (!s_to || !XParseColor(gl_dpy,
			    XDefaultColormap(gl_dpy, gl_scr_info.screen), 
			    s_to, &to))
    return NULL;
  c = from;
  r = from.red; dr = (to.red - from.red) / npixels;
  g = from.green; dg = (to.green - from.green) / npixels;
  b = from.blue; db = (to.blue - from.blue) / npixels;
  pixels = (Pixel *) malloc(sizeof(Pixel) * npixels);
  c.flags = DoRed | DoGreen | DoBlue;
  for (; i < npixels; ++i)
    {
      if (!XAllocColor(gl_dpy, XDefaultColormap(gl_dpy, gl_scr_info.screen),
		       &c))
	got_all = 0;
      pixels[ i ] = c.pixel;
      c.red = (unsigned short) (r += dr);
      c.green = (unsigned short) (g += dg);
      c.blue = (unsigned short) (b += db);
    }
  if (!got_all) 
    {
      char s[256];
      sprintf(s, "color gradient %s to %s\n", s_from, s_to);
    }
  return pixels;
}

Pixel	*AllocNonlinearGradient(s_colors, clen, nsegs, npixels)
     char	*s_colors[];
     int	clen[];
     int	nsegs;
     int	npixels;
{
  Pixel	*pixels;
  int	i;
  int	curpixel;
  int	perc;  
  
  curpixel= 0;
  perc = 0;
  pixels = (Pixel *) malloc(sizeof(Pixel) * npixels);
  if (nsegs < 1) 
    {
      free(pixels);
      return NULL;
    }
  for (i = 0; i < npixels; i++)
    pixels[i] = 0;
  for (i = 0; (i < nsegs) && (curpixel < npixels) && (perc <= 100); ++i) 
    {
      Pixel	*p;
      int	j;
      int	n;
      
      n = clen[i] * npixels / 100;
      p = AllocLinearGradient(s_colors[i], s_colors[i + 1], n);
      if (!p) 
	{
	  free(pixels);
	  return NULL;
	}
      for (j = 0; j < n; ++j)
	pixels[curpixel + j] = p[j];
      perc += clen[i];
      curpixel += n;
      /*
	free(p);
      */
    }
  for (i = curpixel; i < npixels; ++i)
    pixels[i] = pixels[i - 1];
  return pixels;
}

int		gradient(color1, color2, type, target, status)
     char	*color1;
     char	*color2;
     char	*type;
     char	*target;
     char	*status;
{
  char		**s_colors;
  int		nsegs;
  int		i;
  int		sum;
  int		*perc;
  Pixel		*pixels;
  
  sum = 0;
  s_colors = (char **)malloc(sizeof(char *) * 2);
  perc = (int *)malloc(sizeof(int));
  nsegs = 1;
  s_colors[0] = (char *)malloc(sizeof(char) * (strlen(color1) + 1));
  strcpy(s_colors[0], color1);
  s_colors[1] = (char *)malloc(sizeof(char) * (strlen(color2) + 1));
  strcpy(s_colors[1], color2);
  perc[0] = 100;
  for (i = 0; i < nsegs; ++i)
    sum += perc[i];
  if (sum != 100) 
    {
      for (i = 0; i <= nsegs; ++i)
	if (s_colors[i])
	  free(s_colors[i]);
      free(s_colors);
      free(perc);
      return 0;
    }
  pixels = AllocNonlinearGradient(s_colors, perc, nsegs, GRAD_NB_PIXEL);
  /*
    for (i = 0; i <= nsegs; ++i)
    if (s_colors[i])
    free(s_colors[i]);
    free(s_colors);
    free(perc);
  */
  if (!pixels) 
    return 0;	    
  if (!strcmp(target, "titlebar"))
    {
      if (!strcmp("active", status))
	{
	  gl_epiwin.titlebar.pixels_active = pixels;
	  gl_epiwin.titlebar.grad_active_type = *type;
	}
      else
	{
	  gl_epiwin.titlebar.pixels_inactive = pixels;
	  gl_epiwin.titlebar.grad_inactive_type = *type;
	}
    }
  else
    {
      gl_menu.pixels = pixels;
      gl_menu.grad_type = *type;
    }
  return (1);
}

void			draw_gradient(titlebar, target, status)
     Window		titlebar;
     char		*target;
     char		*status;
{
  XRectangle		bounds;
  XWindowAttributes	attrib;
  GC			transmaskgc;
  XGCValues		gcv;
  unsigned long		gcm;
  char			type;
  Pixel			*pixels;
  
  if (!strcmp(target, "titlebar"))
    {
      if (!strcmp("active", status))
	{
	  type = gl_epiwin.titlebar.grad_active_type;
	  pixels = gl_epiwin.titlebar.pixels_active;
	}
      else
	{
	  type = gl_epiwin.titlebar.grad_inactive_type;
	  pixels = gl_epiwin.titlebar.pixels_inactive;
	}
    }
  else
    {
      pixels = gl_menu.pixels;
      type = gl_menu.grad_type;
    }
  gcm = GCFunction | GCPlaneMask | GCGraphicsExposures | GCLineWidth;
  gcv.line_width = 0;
  gcv.function = GXcopy;
  gcv.plane_mask = AllPlanes;
  gcv.graphics_exposures = False;
  transmaskgc = XCreateGC(gl_dpy, gl_scr_info.root_win, gcm, &gcv);
  XGetWindowAttributes(gl_dpy, titlebar, &attrib);
  bounds.x = bounds.y = 0;
  bounds.width = attrib.width;
  bounds.height = attrib.height;
  /*	flush_expose(titlebar); */
  XSetClipMask(gl_dpy, transmaskgc, None);
  if (type == 'H') 
    {
      register int i = 0, dw = bounds.width / GRAD_NB_PIXEL + 1;
      while (i < GRAD_NB_PIXEL)
	{
	  unsigned short x = i * bounds.width / GRAD_NB_PIXEL;
	  XSetForeground(gl_dpy, transmaskgc, pixels[i++]);
	  XFillRectangle(gl_dpy, titlebar, transmaskgc,
			 bounds.x + x, bounds.y,
			 dw, bounds.height);
	}
    } 
  else 
    {
      register int i = 0, dh = bounds.height / GRAD_NB_PIXEL + 1;
      while (i < GRAD_NB_PIXEL)
	{
	  unsigned short y = i * bounds.height / GRAD_NB_PIXEL;
	  XSetForeground(gl_dpy, transmaskgc, pixels[i++]);
	  XFillRectangle(gl_dpy, titlebar, transmaskgc,
			 bounds.x, bounds.y + y,
			 bounds.width, dh);
	}
    }
}



